// There are two lines of thoughts for pointers vs. references.  One
// says to use pointers when something can be null and references
// elsewhere.  The other says to use pass pointers when something is
// mutable and pass const references when it's immutable, because if
// you use non-const reference parameters it's hard to tell at the
// caller that something might be mutated.  This file shows a small
// helper class that might address that issue.  Instead of the caller
// being f(x) for both immutable and mutable parameters, it'll be f(x)
// for immutable and f(receive(x)) for a mutable parameter.  See
// test.cc for an example.

namespace util {
  
  template<typename T>
  struct receivable {
    receivable(T& receiver): receiver_(receiver) {}
    T& get() { return receiver_; }
    operator T& () { return receiver_; }
    T& operator = (const T& v) { receiver_ = v; return receiver_; }
   private:
    T& receiver_;
  };

  template<typename T>
  receivable<T> receive(T& receiver) {
    return receivable<T>(receiver);
  }

}
